=== modified file 'bzrlib/revisiontree.py'
--- bzrlib/revisiontree.py	2006-07-24 02:05:29 +0000
+++ bzrlib/revisiontree.py	2006-07-29 03:57:49 +0000
@@ -113,4 +113,29 @@
     def unlock(self):
         self._repository.unlock()
 
-
+    def walkdirs(self, prefix=""):
+        _directory = 'directory'
+        pending = [('', '', _directory, None, '', None, None)]
+        inv = self.inventory
+        while pending:
+            dirblock = []
+            currentdir = pending.pop()
+            # 0 - relpath, 1- basename, 2- kind, 3- stat, 4-toppath
+            top = currentdir[4]
+            if currentdir[0]:
+                relroot = currentdir[0] + '/'
+            else:
+                relroot = ""
+            # FIXME: stash the node in pending
+            entry = inv[inv.path2id(top)]
+            for name, child in entry.sorted_children():
+                toppath = relroot + name
+                dirblock.append((toppath, name, child.kind, None, toppath,
+                    child.file_id, child.kind
+                    ))
+            yield (currentdir[0], top, entry.file_id), dirblock
+            # push the user specified dirs from dirblock
+            for dir in reversed(dirblock):
+                if dir[2] == _directory:
+                    pending.append(dir)
+ 

=== modified file 'bzrlib/tests/tree_implementations/__init__.py'
--- bzrlib/tests/tree_implementations/__init__.py	2006-07-28 09:00:47 +0000
+++ bzrlib/tests/tree_implementations/__init__.py	2006-07-29 03:57:49 +0000
@@ -224,6 +224,7 @@
     result = TestSuite()
     test_tree_implementations = [
         'bzrlib.tests.tree_implementations.test_test_trees',
+        'bzrlib.tests.tree_implementations.test_walkdirs',
         ]
     adapter = TreeTestProviderAdapter(
         default_transport,

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2006-07-29 01:49:10 +0000
+++ bzrlib/tree.py	2006-07-29 03:57:49 +0000
@@ -157,6 +157,35 @@
         pred = self.inventory.has_filename
         return set((p for p in paths if not pred(p)))
 
+    def walkdirs(self, prefix=""):
+        """Walk the contents of this tree from path down.
+
+        This yields all the data about the contents of a directory at a time.
+        After each directory has been yielded, if the caller has mutated the
+        list to exclude some directories, they are then not descended into.
+        
+        The data yielded is of the form:
+        [(relpath, basename, kind, lstat, path_from_tree_root, file_id, 
+          versioned_kind), ...]
+         - relpath is the relative path within the subtree being walked.
+         - basename is the basename
+         - kind is the kind of the file now. If unknonwn then the file is not
+           present within the tree - but it may be recorded as versioned. See
+           versioned_kind.
+         - lstat is the stat data *if* the file was statted.
+         - path_from_tree_root is the path from the root of the tree.
+         - file_id is the file_id is the entry is versioned.
+         - versioned_kind is the kind of the file as last recorded in the 
+           versioning system. If 'unknown' the file is not versioned.
+        One of 'kind' and 'versioned_kind' must not be 'unknown'.
+
+        :param prefix: Start walking from prefix within the tree rather than
+        at the root. This allows one to walk a subtree but get paths that are
+        relative to a tree rooted higher up.
+        :return: an iterator over the directory data.
+        """
+        raise NotImplementedError(self.walkdirs)
+
 
 # for compatibility
 from bzrlib.revisiontree import RevisionTree

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-07-29 01:49:10 +0000
+++ bzrlib/workingtree.py	2006-07-29 03:57:49 +0000
@@ -1480,6 +1480,32 @@
                              file_id=self.path2id(conflicted)))
         return conflicts
 
+    def walkdirs(self, prefix=""):
+        _directory = 'directory'
+        pending = [('', '', _directory, None, '', None, None)]
+        inv = self.inventory
+        while pending:
+            dirblock = []
+            currentdir = pending.pop()
+            # 0 - relpath, 1- basename, 2- kind, 3- stat, 4-toppath
+            top = currentdir[4]
+            if currentdir[0]:
+                relroot = currentdir[0] + '/'
+            else:
+                relroot = ""
+            # FIXME: stash the node in pending
+            entry = inv[inv.path2id(top)]
+            for name, child in entry.sorted_children():
+                toppath = relroot + name
+                dirblock.append((toppath, name, child.kind, None, toppath,
+                    child.file_id, child.kind
+                    ))
+            yield (currentdir[0], top, entry.file_id), dirblock
+            # push the user specified dirs from dirblock
+            for dir in reversed(dirblock):
+                if dir[2] == _directory:
+                    pending.append(dir)
+
 
 class WorkingTree2(WorkingTree):
     """This is the Format 2 working tree.

